/* -*- mode: c++ -*- */

%module frameCPP

%{
#include "general/types.hh"
#include "framecpp/Detectors.hh"
#include "framecpp/FrDetector.hh"

using namespace FrameCPP;
%}

%import "general/SharedPtr.hh"
%import "framecpp/FrDetector.hh"

%SharedPtr(FrDetector)

class FrDetector
{
public:
  //---------------------------------------------------------------------
  /// \brief Retrieve the name of the instrument.
  ///
  /// \return
  ///     The name of the instrument.
  //---------------------------------------------------------------------
  const std::string& GetName( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the prefix of the instrument.
  ///
  /// \return
  ///     The prefix for the instrument.
  //---------------------------------------------------------------------
  const CHAR* GetPrefix( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the longitude of the detector vertex.
  ///
  /// \return
  ///     The longitude of the detector vertex.
  //---------------------------------------------------------------------
  REAL_8 GetLongitude( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the latitude of the detector vertex.
  ///
  /// \return
  ///     The latitude of the detector vertex.
  //---------------------------------------------------------------------
  REAL_8 GetLatitude( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the vertex elevation of the detector.
  ///
  /// \return
  ///     The vertex elevation of the detector.
  //---------------------------------------------------------------------
  REAL_4 GetElevation( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the orientation of X arm of the detector.
  ///
  /// \return
  ///     The orientation of the X arm of the detector.
  //---------------------------------------------------------------------
  REAL_4 GetArmXazimuth( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the orientation of Y arm of the detector.
  ///
  /// \return
  ///     The orientation of the Y arm of the detector.
  //---------------------------------------------------------------------
  REAL_4 GetArmYazimuth( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the altitude angle of X arm of the detector.
  ///
  /// \return
  ///     The altitude angle of the X arm of the detector.
  //---------------------------------------------------------------------
  REAL_4 GetArmXaltitude( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the altitude angle of Y arm of the detector.
  ///
  /// \return
  ///     The altitude angle of the Y arm of the detector.
  //---------------------------------------------------------------------
  REAL_4 GetArmYaltitude( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the midpoint of the X arm of the detector.
  ///
  /// \return
  ///     The midpoint of the X arm of the detector.
  //---------------------------------------------------------------------
  REAL_4 GetArmXmidpoint( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the midpoint of the Y arm of the detector.
  ///
  /// \return
  ///     The midpoint of the Y arm of the detector.
  //---------------------------------------------------------------------
  REAL_4 GetArmYmidpoint( ) const;

  //---------------------------------------------------------------------
  /// \brief Retrieve the local seasonal time.
  ///
  /// \return
  ///     The local seasonal time.
  //---------------------------------------------------------------------
  INT_4S GetLocalTime( ) const;
};
