#include "framecpp/config.h"

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/Description.hh"
#include "framecpp/Common/SearchContainer.hh"

#include "framecpp/Version3/FrAdcData.hh"
#include "framecpp/Version3/FrSE.hh"
#include "framecpp/Version3/FrSH.hh"

#include "framecpp/Version3/PTR_STRUCT.hh"

using namespace FrameCPP::Version_3;
using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;


using std::ceil;
using std::floor;

//-----------------------------------------------------------------------
// Local functions and variables
//-----------------------------------------------------------------------
static const FrameSpec::Info::frame_object_types s_object_id
= FrameSpec::Info::FSI_FR_ADC_DATA;

//-----------------------------------------------------------------------
// FrAdcData::data_definition_type
//-----------------------------------------------------------------------
FrAdcData::data_definition_type::
data_definition_type( )
{
}

FrAdcData::data_definition_type::
data_definition_type( const data_definition_type& Source )
  : data( Source.data )
{
  copy_core( Source );
}

bool FrAdcData::data_definition_type::
operator==( const data_definition_type& RHS ) const
{
#if LM_INFO
#define LM_INFO_CHECK(a) \
  if ( ! ( a == RHS.a ) ) \
  { \
    std::cerr << "DEBUG: Comparison of " << #a << " failed" << std::endl; \
  }

  LM_INFO_CHECK( name );
  LM_INFO_CHECK( comment);
  LM_INFO_CHECK( crate );
  LM_INFO_CHECK( channel );
  LM_INFO_CHECK( nBits );
  LM_INFO_CHECK( bias );
  LM_INFO_CHECK( slope );
  LM_INFO_CHECK( units );
  LM_INFO_CHECK( fShift );
  LM_INFO_CHECK( timeOffset );
  LM_INFO_CHECK( dataValid );
  LM_INFO_CHECK( phase );
  LM_INFO_CHECK( data );
  LM_INFO_CHECK( aux );
#undef LM_INFO_CHECK
#endif /* LM_INFO */

  const bool	retval =
    ( ( this == &RHS ) ||	// Short circuit for comparison to self
      ( ( name == RHS.name ) &&
	( comment == RHS.comment ) &&
	( crate == RHS.crate ) &&
	( channel == RHS.channel ) &&
	( nBits == RHS.nBits ) &&
	( sampleRate == RHS.sampleRate ) &&
	( bias == RHS.bias ) &&
	( slope == RHS.slope ) &&
	( units == RHS.units ) &&
	( fShift == RHS.fShift ) &&
	( timeOffsetS == RHS.timeOffsetS ) &&
	( timeOffsetN == RHS.timeOffsetN ) &&
	( overRange == RHS.overRange ) &&
	( data == RHS.data ) &&
	( aux == RHS.aux ) ) );

  return retval;

}

bool FrAdcData::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return ( *this == Obj );
}

void FrAdcData::data_definition_type::
copy_core( const data_definition_type& Source )
{
  name = Source.name;
  comment = Source.comment;
  crate = Source.crate;
  channel = Source.channel;
  nBits = Source.nBits;
  sampleRate = Source.sampleRate;
  bias = Source.bias;
  slope = Source.slope;
  units = Source.units;
  fShift = Source.fShift;
  timeOffsetS = Source.timeOffsetS;
  timeOffsetN = Source.timeOffsetN;
  overRange = Source.overRange;
  aux = Source.aux;
}

//-----------------------------------------------------------------------
//
//-----------------------------------------------------------------------

FrAdcData::
FrAdcData( )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
}

FrAdcData::
FrAdcData( const FrAdcData& Source )
  : FrameSpec::Object( s_object_id, StructDescription( ) ),
    m_data( Source.m_data )
{
}

FrAdcData::
FrAdcData( const std::string& name, INT_4U crate,
	   INT_4U channel, INT_4U nBits,
	   REAL_8 sampleRate,
	   REAL_4 bias,
	   REAL_4 slope,
	   const std::string& units,
	   REAL_8 fShift,
	   INT_4S timeOffsetS,
	   INT_4U timeOffsetN,
	   INT_2U overRange )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
  m_data.name = name;
  m_data.crate = crate;
  m_data.channel = channel;
  m_data.nBits = nBits;
  m_data.sampleRate = sampleRate;
  m_data.bias = bias;
  m_data.slope = slope;
  m_data.units = units;
  m_data.fShift = fShift;
  m_data.timeOffsetS = timeOffsetS;
  m_data.timeOffsetN = timeOffsetN;
  m_data.overRange = overRange;
}

FrAdcData::
FrAdcData( IStream& Stream )
  : FrameSpec::Object( s_object_id, StructDescription( ) )
{
    Stream >> m_data.name
	   >> m_data.comment
	   >> m_data.crate
	   >> m_data.channel
	   >> m_data.nBits
	   >> m_data.bias
	   >> m_data.slope
	   >> m_data.units
	   >> m_data.sampleRate
	   >> m_data.timeOffsetS
	   >> m_data.timeOffsetN
	   >> m_data.fShift
	   >> m_data.overRange
	   >> m_data.data
	   >> m_data.aux
      ;
  Stream.Next( this );
}

FrAdcData::
~FrAdcData( )
{
}

const std::string& FrAdcData::
GetName() const
{
   return m_data.name;
}


FrameCPP::cmn_streamsize_type FrAdcData::
Bytes( const Common::StreamBase& Stream ) const
{
  return
    m_data.name.Bytes( ) +
    m_data.comment.Bytes( ) +
    sizeof( m_data.crate ) +
    sizeof( m_data.channel ) +
    sizeof( m_data.nBits ) +
    sizeof( m_data.bias ) +
    sizeof( m_data.slope ) +
    m_data.units.Bytes( ) +
    sizeof( m_data.sampleRate ) +
    sizeof( m_data.timeOffsetS ) +
    sizeof( m_data.timeOffsetN ) +
    sizeof( m_data.fShift ) +
    sizeof( m_data.overRange ) +
    Stream.PtrStructBytes( ) +	// aux
    Stream.PtrStructBytes( ) +	// data
    Stream.PtrStructBytes( )	// next
    ;
}

FrAdcData* FrAdcData::
Create( IStream& Stream ) const
{
  return new FrAdcData( Stream );
}

FrAdcData& FrAdcData::
Merge( const FrAdcData& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += StructName( );

  throw std::domain_error( msg );
  return *this;
}

const char* const FrAdcData::
ObjectStructName( ) const
{
  return StructName( );
}


const Description* FrAdcData::
StructDescription( )
{
  static Description ret;

  if ( ret.size( ) == 0 )
  {
    ret( FrSH( FrAdcData::StructName( ), s_object_id,
	       "AdcData  Data Structure" ) );

    ret( FrSE( "name", "STRING",
	       "" ) );
    ret( FrSE( "comment", "STRING",
	       "" ) );
    ret( FrSE( "crate", "INT_4U",
	       "" ) );
    ret( FrSE( "channel", "INT_4U",
	       "" ) );
    ret( FrSE( "nBits", "INT_4U",
	       "" ) );
    ret( FrSE( "bias", "REAL_4",
	       "" ) );
    ret( FrSE( "slope", "REAL_4",
	       "" ) );
    ret( FrSE( "units", "STRING",
	       "" ) );
    ret( FrSE( "sampleRate", "REAL_8",
	       "" ) );
    ret( FrSE( "timeOffsetS", "INT_4S",
	       "" ) );
    ret( FrSE( "timeOffsetN", "INT_4U",
	       "" ) );
    ret( FrSE( "fShift", "REAL_8",
	       "" ) );
    ret( FrSE( "overRange", "INT_2U",
	       "" ) );

    ret( FrSE( "data", PTR_STRUCT::Desc( FrVect::StructName( ) ),
	       ""  ) );
    ret( FrSE( "aux", PTR_STRUCT::Desc( FrVect::StructName( ) ),
	       ""  ) );

    ret( FrSE( "next",	PTR_STRUCT::Desc( FrAdcData::StructName( ) ),
	       ""  ) );
  }

  return &ret;
}

void FrAdcData::
Write( OStream& Stream ) const
{
  try
  {
    Stream << m_data.name
	   << m_data.comment
	   << m_data.crate
	   << m_data.channel
	   << m_data.nBits
	   << m_data.bias
	   << m_data.slope
	   << m_data.units
	   << m_data.sampleRate
	   << m_data.timeOffsetS
	   << m_data.timeOffsetN
	   << m_data.fShift
	   << m_data.overRange
	   << m_data.data
	   << m_data.aux
      ;
    WriteNext( Stream );
  }
  catch ( const std::exception& Exception )
  {
    std::cerr << "DEBUG: Error Writing FrAdcData: " << m_data.name
	      << " exception: " << Exception.what( )
	      << std::endl;
    throw; // rethrow
  }
}
FrAdcData::demote_ret_type FrAdcData::
demote( INT_2U Target,
	demote_arg_type Obj,
	IStream* Stream ) const
{
  if ( Target >= DATA_FORMAT_VERSION )
  {
    return Obj;
  }
  throw
    Unimplemented( "Object* FrAdcData::demote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

FrAdcData::promote_ret_type FrAdcData::
promote( INT_2U Target, promote_arg_type Obj, IStream* Stream ) const
{
  return Promote( Target, Obj, Stream );
}
